#include "simulation.h"
#include "random.h"
#include <time.h>
#include <stdlib.h>
#include <math.h>
#include <strings.h>

//-------------------------------SHO demo-------------------------------- 
class TService;

CHead     *Queue;                              
TService  *Service;                              

#define POCETPTAKU	100
#define MINX		-499
#define MAXX		499
#define MINR		150
#define MAXR		350

int sx[POCETPTAKU], sy[POCETPTAKU];
int e[POCETPTAKU];
int zpev[POCETPTAKU];

#pragma align=1
typedef struct {
    short x;
    short y;
} ptaci_t;
#pragma align=0

class TPtak : public CProcess                 // request in the queueing system
{          
  virtual void Run();                            // request's behaviour
  public:
  TPtak( int id_);
  u_int id, r;
  int stredx, stredy;
};

int random_xy()
{
    int r = uniform( MINX + 10, MAXX - 10);
    return r;
}

bool goodxy( int xy)
{
    return ((xy >= MINX) && (xy <= MAXX));
}

double vzdalenost( double x1, double y1, double x2, double y2)
{
    double v;
    x1 -= x2;
    y1 -= y2;
    return sqrt( x1*x1 + y1*y1);
}

TPtak::TPtak( int id_)
{
    id = id_;
    sx[id] = stredx = random_xy();
    sy[id] = stredy = random_xy();
    r = uniform( MINR, MAXR);
    e[id] = 0;
    zpev[id] = 0;
//    printf("Uzel %d: stred=(%d,%d) r=%d\n", id, stredx, stredy, r);
}

void TPtak::Run()
{
    double v, uhel, polomer, dx, dy, ax, ay;
    int cilx, cily, i, cekat;
    
    while (1) {
	Hold(1);
	do {
		uhel = uniform( 0, 2*M_PI);
		polomer = normal( r/3.0f, r/3.0f*2.0f);
		cilx = stredx + polomer * sin( uhel);
		cily = stredy + polomer * cos( uhel);
	} while ((!goodxy( cilx)) || (!goodxy( cily)));

	ax = sx[id];
	ay = sy[id];
	while (1) {
		v = vzdalenost( sx[id], sy[id], cilx, cily);
		if (v < 2) break;
		ax += (cilx - ax) / v * 2;
		ay += (cily - ay) / v * 2;
		sx[id] = (int) ax;
		sy[id] = (int) ay;
		Hold(1);
	}
	sx[id] = cilx;
	sy[id] = cily;
	Hold(1);
	if (uniform(0,100) < 1) { e[id] = 1; zpev[id]++; }
	cekat = uniform( 2, 9);
	for (i=0; i<cekat; i++) { Hold(1); e[id] = 0; }
    }
}

/*----------------------------------------------*/
void CSimulation::Run()                          // main process
{
  int i,j;
  FILE *f;
  ptaci_t pozice;
  int lastx[POCETPTAKU], lasty[POCETPTAKU];
  int dx, dy;
  unsigned char pack, bitarr[256/8];
  unsigned long delkasimulace = 200000;
  
  for (i=0; i<POCETPTAKU; i++) {
      (new TPtak(i))->ActivateAt(Time ());           // creating and scheduling 1st request
  }

  f = fopen("ptaci.bin","wb");
  if (f == NULL) {
      printf("fopen error\n");
      exit(1);
  }
    
  Hold(1);

  fwrite( &delkasimulace, sizeof( delkasimulace), 1, f);
  for (j=0; j<POCETPTAKU; j++) {
      pozice.x = sx[j];
      pozice.y = sy[j];
      fwrite( &pozice, sizeof(pozice), 1, f);
      lastx[j] = sx[j];
      lasty[j] = sy[j];
  }

  Hold(1);

  for (i=0; i<delkasimulace; i++) {
    if ((i % 1024) == 0) {
	printf("cas: %d\n", i);
    }
    for (j=0; j<POCETPTAKU; j++) {
	pozice.x = sx[j] - lastx[j] + 4;
	pozice.y = sy[j] - lasty[j] + 4;
	if ((pozice.x < 0) || (pozice.x > 6)) printf("err: dx=%d  ptak=%d  x: %d -> %d\n", pozice.x, j, lastx[j], sx[j]);
	if ((pozice.y < 0) || (pozice.y > 6)) printf("err: dy=%d  ptak=%d  y: %d -> %d\n", pozice.y, j, lasty[j], sy[j]);
	pack = pozice.x | (pozice.y << 4);

	if (e[j]) {
//	    printf("udalost ptak=%d\n", j);
	    if (pack != ((4<<4)+4)) printf("err: udalost, ale dx=%d dy=%d ptak=%d\n", pozice.x, pozice.y, j);
	    pack = 0xFF;
	}

	fwrite( &pack, sizeof(pack), 1, f);
	
        lastx[j] = sx[j];
        lasty[j] = sy[j];
    }
    Hold(1);
  }
  
  for (j=0; j<POCETPTAKU; j++) {
      printf("zpev ptaka %d: %d\n", j, zpev[j]);
  }
  
  fclose(f);
  exit(0);
};
