//---------------------------------------------------------------------
// Vizualizace trajektorie ptaku pro X36DSI, LS2010
// Autor: Jiri Smitka <jiri.smitka@gmail.com>
//
// Program byl napsan podle "Ukazkoveho prikladu cislo 2" v serialu
// "Grafická knihovna OpenGL" publikovaneho v roce 2003 na root.cz,
// autor: Pavel Tisnovsky
//
// Program otevre jedno hlavni okno a vykresli do nej graficka primitiva
// podporovana OpenGL. Okno (a tim i celou aplikaci) lze ukoncit stiskem
// klavesy ESC.
//---------------------------------------------------------------------

#include <GL/glut.h>                            // hlavickovy soubor funkci GLUTu
#include <math.h>

#include "ptaci.h"

short *px, *py;
int delkasimulace;
char vsechno = 0;
char running = 1;
int cas = 0;
double barva_r[POCETPTAKU], barva_g[POCETPTAKU], barva_b[POCETPTAKU];

#define TIMER_MS  25

double uprav( double x)
{
    return (x + 499) / 2.0f;
}

double getx( int ptak, int cas)
{
    return *(px + (POCETPTAKU * cas) + ptak);
}

double gety( int ptak, int cas)
{
    return *(py + (POCETPTAKU * cas) + ptak);
}

double vzdalenost( int ptak1, int ptak2, int cas)
{
    double x = getx( ptak1, cas) - (ptak2 == -1 ? 0 : getx( ptak2, cas));
    double y = gety( ptak1, cas) - (ptak2 == -1 ? 0 : gety( ptak2, cas));
    return sqrt( x*x + y*y);
}

void kruznice( int ptak, int cas)
{
    double uhel, x, y;

    if (ptak == -1) {
	x = 0;
	y = 0;
	glColor3f(1,1,1);
    }
    else {
	x = getx( ptak, cas);
	y = gety( ptak, cas);
	glColor3f( barva_r[ptak], barva_g[ptak], barva_b[ptak]);
    }
    glBegin(GL_LINE_STRIP);                     // nyni vykreslime polycaru
    uhel = 0;
    while (uhel < 2*M_PI) {
	glVertex2i( uprav( x + 20 * sin( uhel)), uprav( y + 20 * cos( uhel)));
	uhel += M_PI/30;
    }
    glVertex2i( uprav(x), uprav(y + 20));
    glEnd();
}

//---------------------------------------------------------------------
// Nastaveni souradneho systemu v zavislosti na velikosti okna
//---------------------------------------------------------------------
void onResize(int w, int h)                     // w a h reprezentuje novou velikost okna
{
    glViewport(0, 0, w, h);                     // viditelna oblast pres cele okno
    glMatrixMode(GL_PROJECTION);                // zacatek modifikace projekcni matice
    glLoadIdentity();                           // vymazani projekcni matice (=identita)
    glOrtho(0, w, 0, h, -1, 1);                 // mapovani abstraktnich souradnic do souradnic okna
}



//---------------------------------------------------------------------
// Tato funkce je volana pri kazdem prekresleni okna
//---------------------------------------------------------------------
void onDisplay(void)
{
    int t, ptak, jinyptak, px, py;
    char vdosahu, zakladna;
    
    glClearColor(0.0, 0.0, 0.0, 0.0);           // nastaveni mazaci barvy na cernou
    glClear(GL_COLOR_BUFFER_BIT);               // vymazani bitovych rovin barvoveho bufferu

    if (vsechno)
    {
	for (ptak = 0; ptak < POCETPTAKU; ptak++)
	{
	    glColor3f( barva_r[ptak], barva_g[ptak], barva_b[ptak]);
	    glBegin(GL_LINE_STRIP);                     // nyni vykreslime polycaru
	    for (t=0; t<delkasimulace; t++) {
		glVertex2i( uprav( getx( ptak, t)), uprav( gety( ptak, t)));
	    }
	    glEnd();
        }
    }
    else
    {
	zakladna = 0;
	for (ptak = 0; ptak < POCETPTAKU; ptak++)
	{
	    px = uprav( getx( ptak, cas));
	    py = uprav( gety( ptak, cas));
	    glColor3f( barva_r[ptak], barva_g[ptak], barva_b[ptak]);
	    glBegin(GL_QUADS);                     // nyni vykreslime polycaru
	    glVertex2i( px - 1, py - 1);
	    glVertex2i( px - 1, py + 1);
	    glVertex2i( px + 1, py + 1);
	    glVertex2i( px + 1, py - 1);
	    glEnd();

	    vdosahu = 0;
	    for (jinyptak = ptak + 1; jinyptak < POCETPTAKU; jinyptak++)
	    {
		  if (vzdalenost( ptak, jinyptak, cas) <= 20) {
			vdosahu = 1;
			kruznice( jinyptak, cas);
		  }
	    }
	    if (vdosahu) kruznice( ptak, cas);
	    
	    if (vzdalenost( ptak, -1, cas) <= 20) zakladna = 1;
	}
	if (zakladna) kruznice( -1, 0);
    }

    // zakladna
    glColor3f( 1, 1, 1);
    glBegin(GL_QUADS);                     // nyni vykreslime polycaru
    glVertex2i( uprav(0)-1, uprav(0)-1);
    glVertex2i( uprav(0)+1, uprav(0)-1);
    glVertex2i( uprav(0)+1, uprav(0)+1);
    glVertex2i( uprav(0)-1, uprav(0)+1);
    glEnd();
    
    glFlush();                                  // provedeni a vykresleni zmen
}


void onTimer(int x)
{
  if (running) {
      if (cas++ >= delkasimulace-1) {
	    cas = 0;
	    running = 0;
      }
      glutPostRedisplay();                  			// a rekneme, ze ma dojit k prekresleni
      if (running) glutTimerFunc( TIMER_MS, onTimer, 0);	// naplanujeme, ze za dalsich 25ms se ma funkce zavolat
  }
}
 

//---------------------------------------------------------------------
// Tato funkce je volana pri stlaceni ASCII klavesy
//---------------------------------------------------------------------
void onKeyboard(unsigned char key, int x, int y)
{
    switch (key) {
      case 27:	{ exit(0); }
      case 8:	{
		    if (vsechno) return;
		    running = 0;
		    if (cas > 0) cas--;
		    break;
      }
      case 32:	{
		    if (vsechno) return;
		    running = !running;
		    if (running) glutTimerFunc( TIMER_MS, onTimer, 0);
		    break;
      }
      case 13:  {
		    running = 0;
		    vsechno = !vsechno;
		    break;
      }
    }
    glutPostRedisplay();                    // a prekreslit scenu
//    printf("%d\n", key);
}

//---------------------------------------------------------------------
// Hlavni funkce konzolove aplikace
//---------------------------------------------------------------------
int main(int argc, char **argv)
{
    int t, ptak, n=0;
    double r,g,b;
    
    delkasimulace = zjisti_delku_simulace();
    printf("Delka simulace: %d sekund\n", delkasimulace);
    px = (short *) malloc( delkasimulace * POCETPTAKU * sizeof(short));
    py = (short *) malloc( delkasimulace * POCETPTAKU * sizeof(short));
    double total = (POCETPTAKU * delkasimulace) / 100.0f;
    for (t = 0; t < delkasimulace; t++) {
        if (!nacti_dalsi_zaznamy()) {
		fprintf( stderr, "Nekde je chyba.\n");
		exit(1);
	}
	for (ptak=0; ptak < POCETPTAKU; ptak++) {
	    if ((n % 4096) == 0) {
		 printf("\rNacitam data: %.0f%% ", n / total);
		 fflush( stdout);
	    }
	    px[n] = ptaci_x[ptak];
	    py[n++] = ptaci_y[ptak];
	}
    }
    printf("\rData nactena, spoustim vykreslovani...\n");

    r = 0.2;
    g = 0.2;
    b = 0.2;
    for (ptak = 0; ptak < POCETPTAKU; ptak++)
    {
	barva_r[ptak] = r;
	barva_g[ptak] = g;
	barva_b[ptak] = b;
	r += 0.2;
	if (r >= 1.0) {
	    r = 0.2;
	    g += 0.2;
	    if (g >= 1.0) {
		g = 0.2;
		b += 0.12;
		if (b > 1.0) printf("Problem s poctem barev.\n");
	    }
	}
    }
    
    
    glutInit(&argc, argv);                      // inicializace knihovny GLUT
    glutCreateWindow("Ptaci");			// vytvoreni okna pro kresleni
    glutReshapeWindow(500, 500);                // zmena velikosti okna
    glutPositionWindow(100, 100);               // pozice leveho horniho rohu okna
    glutDisplayFunc(onDisplay);                 // registrace funkce volane pri prekreslovani okna
    glutReshapeFunc(onResize);                  // registrace funkce volane pri zmene velikosti okna
    glutKeyboardFunc(onKeyboard);               // registrace funkce volani pri stlaceni klavesy
    glutTimerFunc( TIMER_MS, onTimer, 0);	// registrace funkce volane za 25 milisekund
    glutMainLoop();                             // nekonecna smycka, kde se volaji zaregistrovane funkce
    return 0;                                   // navratova hodnota vracena operacnimu systemu
}



//---------------------------------------------------------------------
// Konec zdrojoveho souboru
//---------------------------------------------------------------------

    